@@ -17,6 +17,7 @@ package org.uberfire.ext.wires.core.grids.client.widget.dnd;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Optional;
 
 import com.ait.lienzo.client.core.event.NodeMouseMoveEvent;
 import com.ait.lienzo.client.core.event.NodeMouseMoveHandler;
@@ -27,6 +28,7 @@ import com.google.gwt.dom.client.Style;
 import org.uberfire.ext.wires.core.grids.client.model.GridColumn;
 import org.uberfire.ext.wires.core.grids.client.model.GridData;
 import org.uberfire.ext.wires.core.grids.client.model.GridRow;
+import org.uberfire.ext.wires.core.grids.client.util.ColumnIndexUtilities;
 import org.uberfire.ext.wires.core.grids.client.util.CoordinateUtilities;
 import org.uberfire.ext.wires.core.grids.client.widget.dom.HasDOMElementResources;
 import org.uberfire.ext.wires.core.grids.client.widget.grid.GridWidget;
@@ -98,10 +100,15 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
 
             final Group header = gridWidget.getHeader();
             final GridRenderer renderer = gridWidget.getRenderer();
-
             final double headerHeight = renderer.getHeaderHeight();
-            final double headerRowsYOffset = getHeaderRowsYOffset(gridWidget,
-                                                                  renderer);
+
+            final BaseGridRendererHelper rendererHelper = gridWidget.getRendererHelper();
+            final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
+            if (renderingInformation == null) {
+                continue;
+            }
+
+            final double headerRowsYOffset = renderingInformation.getHeaderRowsYOffset();
             final double headerMinY = (header == null ? headerRowsYOffset : header.getY() + headerRowsYOffset);
             final double headerMaxY = (header == null ? headerHeight : headerHeight + header.getY());
 
@@ -161,18 +168,6 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
         }
     }
 
-    private double getHeaderRowsYOffset(final GridWidget gridWidget,
-                                        final GridRenderer renderer) {
-        final GridData model = gridWidget.getModel();
-        final int headerRowCount = model.getHeaderRowCount();
-        final double headerHeight = renderer.getHeaderHeight();
-        final double headerRowHeight = renderer.getHeaderRowHeight();
-        final double headerRowsHeight = headerRowHeight * headerRowCount;
-        final double headerRowsYOffset = headerHeight - headerRowsHeight;
-
-        return headerRowsYOffset;
-    }
-
     private void setCursor(final Style.Cursor cursor) {
         for (IMediator mediator : layer.getViewport().getMediators()) {
             if (mediator instanceof RestrictedMousePanMediator) {
@@ -316,14 +311,14 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                                                 final List<GridColumn.HeaderMetaData> headerMetaData,
                                                 final int headerRowIndex,
                                                 final int headerColumnIndex) {
-        final int blockStartColumnIndex = getBlockStartColumnIndex(allColumns,
-                                                                   headerMetaData.get(headerRowIndex),
-                                                                   headerRowIndex,
-                                                                   headerColumnIndex);
-        final int blockEndColumnIndex = getBlockEndColumnIndex(allColumns,
-                                                               headerMetaData.get(headerRowIndex),
-                                                               headerRowIndex,
-                                                               headerColumnIndex);
+        final int blockStartColumnIndex = ColumnIndexUtilities.getHeaderBlockStartColumnIndex(allColumns,
+                                                                                              headerMetaData.get(headerRowIndex),
+                                                                                              headerRowIndex,
+                                                                                              headerColumnIndex);
+        final int blockEndColumnIndex = ColumnIndexUtilities.getHeaderBlockEndColumnIndex(allColumns,
+                                                                                          headerMetaData.get(headerRowIndex),
+                                                                                          headerRowIndex,
+                                                                                          headerColumnIndex);
 
         final List<GridColumn<?>> columns = new ArrayList<GridColumn<?>>();
         columns.addAll(allColumns.subList(blockStartColumnIndex,
@@ -331,56 +326,6 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
         return columns;
     }
 
-    @SuppressWarnings("unchecked")
-    private int getBlockStartColumnIndex(final List<? extends GridColumn<?>> allColumns,
-                                         final GridColumn.HeaderMetaData headerMetaData,
-                                         final int headerRowIndex,
-                                         final int headerColumnIndex) {
-        //Back-track adding width of proceeding columns sharing header MetaData
-        int candidateHeaderColumnIndex = headerColumnIndex;
-        if (candidateHeaderColumnIndex == 0) {
-            return candidateHeaderColumnIndex;
-        }
-        while (candidateHeaderColumnIndex > 0) {
-            final GridColumn candidateColumn = allColumns.get(candidateHeaderColumnIndex - 1);
-            final List<GridColumn.HeaderMetaData> candidateHeaderMetaData = candidateColumn.getHeaderMetaData();
-            if (candidateHeaderMetaData.size() - 1 < headerRowIndex) {
-                break;
-            }
-            if (!candidateHeaderMetaData.get(headerRowIndex).equals(headerMetaData)) {
-                break;
-            }
-            candidateHeaderColumnIndex--;
-        }
-
-        return candidateHeaderColumnIndex;
-    }
-
-    @SuppressWarnings("unchecked")
-    private int getBlockEndColumnIndex(final List<? extends GridColumn<?>> allColumns,
-                                       final GridColumn.HeaderMetaData headerMetaData,
-                                       final int headerRowIndex,
-                                       final int headerColumnIndex) {
-        //Forward-track adding width of following columns sharing header MetaData
-        int candidateHeaderColumnIndex = headerColumnIndex;
-        if (candidateHeaderColumnIndex == allColumns.size() - 1) {
-            return candidateHeaderColumnIndex;
-        }
-        while (candidateHeaderColumnIndex < allColumns.size() - 1) {
-            final GridColumn candidateColumn = allColumns.get(candidateHeaderColumnIndex + 1);
-            final List<GridColumn.HeaderMetaData> candidateHeaderMetaData = candidateColumn.getHeaderMetaData();
-            if (candidateHeaderMetaData.size() - 1 < headerRowIndex) {
-                break;
-            }
-            if (!candidateHeaderMetaData.get(headerRowIndex).equals(headerMetaData)) {
-                break;
-            }
-            candidateHeaderColumnIndex++;
-        }
-
-        return candidateHeaderColumnIndex;
-    }
-
     protected void findMovableRows(final GridWidget view,
                                    final double cx,
                                    final double cy) {
@@ -498,11 +443,56 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                 columnNewWidth = columnMaximumWidth;
             }
         }
+
         destroyColumns(allGridColumns);
-        activeGridColumn.setWidth(columnNewWidth);
+
+        activeGridColumn.setWidth(adjustColumnWidth(columnNewWidth, activeGridColumn, activeGridWidget));
         layer.batch();
     }
 
+    protected double adjustColumnWidth(double columnNewWidth, GridColumn<?> activeGridColumn, GridWidget activeGridWidget) {
+        final GridData activeGridModel = activeGridWidget.getModel();
+
+        double originalLeftColumnWidth = activeGridColumn.getWidth();
+        double delta = originalLeftColumnWidth - columnNewWidth;
+
+        int visibleWidth = activeGridModel.getVisibleWidth();
+        double gridWidgetWidth = activeGridWidget.getWidth();
+        double newGridWidth = gridWidgetWidth - delta;
+
+        // if the grid is becoming less than 100% width
+        if (newGridWidth < visibleWidth && delta > 0) {
+
+            // look for a column with AUTO width on the right
+            Optional<GridColumn<?>> rightGridColumn = getFirstRightAutoColumn(activeGridColumn, activeGridModel);
+            if (rightGridColumn.isPresent()) {
+                GridColumn<?> rightColumn = rightGridColumn.get();
+                double originalRightColumnWidth = rightColumn.getWidth();
+                double newWidth = originalRightColumnWidth + delta;
+
+                rightColumn.setWidth(newWidth);
+            }
+            // or revert column resizing if the column itself has AUTO width
+            else if (GridColumn.ColumnWidthMode.isAuto(activeGridColumn)) {
+                columnNewWidth = originalLeftColumnWidth;
+            }
+        }
+        return columnNewWidth;
+    }
+
+    Optional<GridColumn<?>> getFirstRightAutoColumn(GridColumn<?> target, GridData model) {
+        List<GridColumn<?>> columns = model.getColumns();
+        int targetIndex = columns.indexOf(target);
+
+        for (int i = targetIndex + 1; i < columns.size(); i += 1) {
+            GridColumn<?> gridColumn = columns.get(i);
+            if (GridColumn.ColumnWidthMode.isAuto(gridColumn)) {
+                return Optional.of(gridColumn);
+            }
+        }
+        return Optional.empty();
+    }
+
     @SuppressWarnings("unchecked")
     protected void handleColumnMove(final NodeMouseMoveEvent event) {
         final GridWidget activeGridWidget = state.getActiveGridWidget();
@@ -540,14 +530,14 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                     for (int headerRowIndex = 0; headerRowIndex < candidateGridColumn.getHeaderMetaData().size(); headerRowIndex++) {
                         final GridColumn.HeaderMetaData candidateHeaderMetaData = candidateGridColumn.getHeaderMetaData().get(headerRowIndex);
                         if (candidateHeaderMetaData.getColumnGroup().equals(activeHeaderMetaData.getColumnGroup())) {
-                            final int candidateBlockStartColumnIndex = getBlockStartColumnIndex(allGridColumns,
-                                                                                                candidateHeaderMetaData,
-                                                                                                headerRowIndex,
-                                                                                                headerColumnIndex);
-                            final int candidateBlockEndColumnIndex = getBlockEndColumnIndex(allGridColumns,
-                                                                                            candidateHeaderMetaData,
-                                                                                            headerRowIndex,
-                                                                                            headerColumnIndex);
+                            final int candidateBlockStartColumnIndex = ColumnIndexUtilities.getHeaderBlockStartColumnIndex(allGridColumns,
+                                                                                                                           candidateHeaderMetaData,
+                                                                                                                           headerRowIndex,
+                                                                                                                           headerColumnIndex);
+                            final int candidateBlockEndColumnIndex = ColumnIndexUtilities.getHeaderBlockEndColumnIndex(allGridColumns,
+                                                                                                                       candidateHeaderMetaData,
+                                                                                                                       headerRowIndex,
+                                                                                                                       headerColumnIndex);
                             final double candidateBlockOffset = rendererHelper.getColumnOffset(candidateBlockStartColumnIndex);
                             final double candidateBlockWidth = getBlockWidth(allGridColumns,
                                                                              candidateBlockStartColumnIndex,
@@ -563,14 +553,14 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                                     destroyColumns(allGridColumns);
                                     activeGridModel.moveColumnsTo(candidateBlockEndColumnIndex,
                                                                   activeGridColumns);
-                                    state.getEventColumnHighlight().setX(activeGridWidget.getAbsoluteX() + rendererHelper.getColumnOffset(activeGridColumns.get(0)));
+                                    state.getEventColumnHighlight().setX(activeGridWidget.getComputedLocation().getX() + rendererHelper.getColumnOffset(activeGridColumns.get(0)));
                                     layer.batch();
                                     return;
                                 } else {
                                     destroyColumns(allGridColumns);
                                     activeGridModel.moveColumnsTo(candidateBlockStartColumnIndex,
                                                                   activeGridColumns);
-                                    state.getEventColumnHighlight().setX(activeGridWidget.getAbsoluteX() + rendererHelper.getColumnOffset(activeGridColumns.get(0)));
+                                    state.getEventColumnHighlight().setX(activeGridWidget.getComputedLocation().getX() + rendererHelper.getColumnOffset(activeGridColumns.get(0)));
                                     layer.batch();
                                     return;
                                 }
@@ -650,7 +640,7 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                                    activeGridRows);
 
         final double rowOffsetY = rendererHelper.getRowOffset(leadRow) + headerHeight;
-        state.getEventColumnHighlight().setY(activeGridWidget.getAbsoluteY() + rowOffsetY);
+        state.getEventColumnHighlight().setY(activeGridWidget.getComputedLocation().getY() + rowOffsetY);
         layer.batch();
     }
 
