@@ -17,6 +17,7 @@
 package org.uberfire.ext.wires.core.grids.client.widget.dnd;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
@@ -25,6 +26,7 @@ import com.ait.lienzo.client.core.event.INodeXYEvent;
 import com.ait.lienzo.client.core.event.NodeMouseMoveEvent;
 import com.ait.lienzo.client.core.mediator.Mediators;
 import com.ait.lienzo.client.core.shape.Viewport;
+import com.ait.lienzo.client.core.types.Point2D;
 import com.ait.lienzo.test.LienzoMockitoTestRunner;
 import com.google.gwt.dom.client.DivElement;
 import com.google.gwt.dom.client.Style;
@@ -48,9 +50,9 @@ import org.uberfire.ext.wires.core.grids.client.widget.grid.renderers.columns.Gr
 import org.uberfire.ext.wires.core.grids.client.widget.grid.renderers.grids.GridRenderer;
 import org.uberfire.ext.wires.core.grids.client.widget.grid.renderers.grids.impl.BaseGridRendererHelper;
 import org.uberfire.ext.wires.core.grids.client.widget.layer.GridLayer;
-import org.uberfire.mvp.Command;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
@@ -93,6 +95,9 @@ public class GridWidgetDnDMouseMoveHandlerTest {
     @Mock
     private NodeMouseMoveEvent event;
 
+    @Mock
+    private BaseHeaderMetaData header;
+
     @Captor
     private ArgumentCaptor<List<GridColumn<?>>> uiColumnsArgumentCaptor;
 
@@ -110,15 +115,18 @@ public class GridWidgetDnDMouseMoveHandlerTest {
 
     private GridWidgetDnDMouseMoveHandler handler;
 
+    private double originalColumnWidth = 200;
+    private double originalRightColumnWidth = 300;
+    private BaseGridColumn<String> column;
+    private BaseGridColumn<String> rightColumn;
+
     @Before
     public void setup() {
         this.uiColumn1 = new RowNumberColumn();
-        this.uiColumn2 = new BaseGridColumn<>(new BaseHeaderMetaData("title"),
+        this.uiColumn2 = new BaseGridColumn<>(Arrays.asList(new BaseHeaderMetaData("title1"), new BaseHeaderMetaData("title2")),
                                               columnRenderer,
                                               100.0);
-        this.uiModel = new BaseGridData() {{
-            setHeaderRowCount(2);
-        }};
+        this.uiModel = new BaseGridData();
         uiModel.appendColumn(uiColumn1);
         uiModel.appendColumn(uiColumn2);
         uiModel.appendRow(new BaseGridRow());
@@ -140,8 +148,9 @@ public class GridWidgetDnDMouseMoveHandlerTest {
         when(gridWidget.getRendererHelper()).thenReturn(helper);
         when(gridWidget.getWidth()).thenReturn(150.0);
         when(gridWidget.getHeight()).thenReturn(124.0);
-        when(gridWidget.getAbsoluteX()).thenReturn(100.0);
-        when(gridWidget.getAbsoluteY()).thenReturn(100.0);
+
+        final Point2D computedLocation = new Point2D(100.0, 100.0);
+        when(gridWidget.getComputedLocation()).thenReturn(computedLocation);
 
         final BaseGridRendererHelper.RenderingInformation ri = new BaseGridRendererHelper.RenderingInformation(mock(Bounds.class),
                                                                                                                uiModel.getColumns(),
@@ -164,8 +173,9 @@ public class GridWidgetDnDMouseMoveHandlerTest {
                                                                                                                }},
                                                                                                                false,
                                                                                                                false,
-                                                                                                               0,
                                                                                                                2,
+                                                                                                               0,
+                                                                                                               0,
                                                                                                                0);
         when(helper.getRenderingInformation()).thenReturn(ri);
 
@@ -175,6 +185,9 @@ public class GridWidgetDnDMouseMoveHandlerTest {
         final GridWidgetDnDMouseMoveHandler wrapped = new GridWidgetDnDMouseMoveHandler(layer,
                                                                                         state);
         this.handler = spy(wrapped);
+
+        column = new BaseGridColumn<>(header, columnRenderer, originalColumnWidth);
+        rightColumn = new BaseGridColumn<>(header, columnRenderer, originalRightColumnWidth);
     }
 
     @Test
@@ -318,7 +331,7 @@ public class GridWidgetDnDMouseMoveHandlerTest {
     }
 
     private void doFindMovableGridWhenOverDragHandle(final boolean isPinned,
-                                                     final Command assertion) {
+                                                     final Runnable assertion) {
         state.setOperation(GridWidgetHandlersOperation.NONE);
         when(gridWidget.isVisible()).thenReturn(true);
         when(gridWidget.onDragHandle(any(INodeXYEvent.class))).thenReturn(true);
@@ -336,7 +349,7 @@ public class GridWidgetDnDMouseMoveHandlerTest {
         verify(handler,
                times(1)).findGridColumn(eq(event));
 
-        assertion.execute();
+        assertion.run();
     }
 
     @Test
@@ -470,4 +483,40 @@ public class GridWidgetDnDMouseMoveHandlerTest {
                      uiRows.size());
         assertTrue(uiRows.contains(uiModel.getRow(0)));
     }
+
+    @Test
+    public void adjustColumnWidth() {
+        double proposedNewWidth = 100;
+        uiModel.setVisibleSizeAndRefresh(10, 0);
+        assertEquals(proposedNewWidth, handler.adjustColumnWidth(proposedNewWidth, column, gridWidget), 0.1);
+
+        uiModel.setVisibleSizeAndRefresh(1000, 0);
+        proposedNewWidth = 300;
+        assertEquals(proposedNewWidth, handler.adjustColumnWidth(proposedNewWidth, column, gridWidget), 0.1);
+
+        proposedNewWidth = 100;
+        column.setColumnWidthMode(GridColumn.ColumnWidthMode.AUTO);
+        assertEquals(originalColumnWidth, handler.adjustColumnWidth(proposedNewWidth, column, gridWidget), 0.1);
+
+        column.setColumnWidthMode(GridColumn.ColumnWidthMode.FIXED);
+        uiModel.appendColumn(column);
+        uiModel.appendColumn(rightColumn);
+        rightColumn.setColumnWidthMode(GridColumn.ColumnWidthMode.AUTO);
+        assertEquals(proposedNewWidth, handler.adjustColumnWidth(proposedNewWidth, column, gridWidget), 0.1);
+        assertEquals(originalRightColumnWidth + (originalColumnWidth - proposedNewWidth), rightColumn.getWidth(), 0.1);
+    }
+
+    @Test
+    public void getFirstRightAutoColumn() {
+        assertFalse(handler.getFirstRightAutoColumn(column, uiModel).isPresent());
+
+        uiModel.appendColumn(column);
+        assertFalse(handler.getFirstRightAutoColumn(column, uiModel).isPresent());
+
+        uiModel.appendColumn(rightColumn);
+        assertFalse(handler.getFirstRightAutoColumn(column, uiModel).isPresent());
+
+        rightColumn.setColumnWidthMode(GridColumn.ColumnWidthMode.AUTO);
+        assertEquals(rightColumn, handler.getFirstRightAutoColumn(column, uiModel).get());
+    }
 }
